# Appcelerator API Docs Spy

This NPM package allows you to query JS(ON) versions of Appcelerator API docs such as for [Titanium](http://docs.appcelerator.com/titanium/data/index.html) and [Alloy](http://docs.appcelerator.com/titanium/data/alloy_index.html) using [JSPath](https://github.com/dfilatov/jspath) and a simplified custom [Path](#path) syntax.

## Install

```bash
$ npm install appc-apidocs --save
```

## Usage

### Module
Use the module API as follows:

```javascript
var spy = require('appc-apispy');

// load an object or a JS or JSON file with docs
spy.load('/path/to/docs.json', function(err, docs) {

	if (err) {
		return console.error(err);
	}

	// query the docs
	docs.query('."Ti.UI.View"', {
	
		// optional, since:
		// queries following . will default to 'jspath'
		// queries following / will default to 'path'
		ql: 'jspath',

	}, function(err, res) {
		
		if (err) {
			return console.error(err);
		}

		// should be an array of one
		console.log(res);
		
		// release the (huge) docs from memory
		docs.release();

	});
	
});
```

Or if you only need to run one query on the same doc that will be released right away:

```javascript
var spy = require('appc-apispy');

// load an object or a JS or JSON file with docs
spy.query('/path/to/docs.json', '$."Ti.UI.View"', function(err, res) {
	
	if (err) {
		return console.error(err);
	}

	// should be an array of one
	console.log(res);
});
```

### REPL
For easy testing and playing, it includes a REPL:

```bash
$ npm run repl

Usage:	query <ql> <query>	Run a query using the given QL.
		load <path>			Load the given API Docs JS/JSON file.
		help				Display this help.
```

If you've cloned the repo, you can load the 3.5.0 Titanium API:

```bash
$ npm run repl
> load test/fixtures/3.5.0.js
```

## Examples queries

All properties of any object with support for Android

```
.*.properties{.platforms.name === "android"}
```

Names of all objects with support on iOS:

```
.*{.platforms.name==="iphone"}.name
```
	
### Shortcuts
JSPath will automatically expand some shortcuts:

```
."Ti.UI.View"
."Titanium.UI.View"
```

```
.'Titanium.UI.View'	
."Titanium.UI.View"
```

```
."Titanium.UI.View.backgroundColor"
Ti.UI.View(.properties|.events|.methods).{.name==="backgroundColor"}
```

### Path syntax
The custom 'path' syntax is a wrapper arround JSPath.

```
/Ti.UI.View.backgroundColor/*/platforms//name
."Ti.UI.View.backgroundColor".*.platforms..name
```

# Contributing

This project is open source and licensed under the [Apache Public License (version 2)](http://www.apache.org/licenses/LICENSE-2.0).  Please consider forking this project to improve, enhance or fix issues. If you feel like the community will benefit from your fork, please open a pull request. 

To protect the interests of the contributors, Appcelerator, customers and end users we require contributors to sign a Contributors License Agreement (CLA) before we pull the changes into the main repository. Our CLA is simple and straightforward - it requires that the contributions you make to any Appcelerator open source project are properly licensed and that you have the legal authority to make those changes. This helps us significantly reduce future legal risk for everyone involved. It is easy, helps everyone, takes only a few minutes, and only needs to be completed once. 

[You can digitally sign the CLA](http://bit.ly/app_cla) online. Please indicate your email address in your first pull request so that we can make sure that will locate your CLA.  Once you've submitted it, you no longer need to send one for subsequent submissions.

# Legal Stuff

Appcelerator is a registered trademark of Appcelerator, Inc. Arrow and associated marks are trademarks of Appcelerator. All other marks are intellectual property of their respective owners. Please see the LEGAL information about using our trademarks, privacy policy, terms of usage and other legal information at [http://www.appcelerator.com/legal](http://www.appcelerator.com/legal).
