#!/usr/bin/env node

var repl = require('repl'),
	path = require('path'),
	util = require('util'),
	spy = require('../lib');

var docs;

var USAGE = 'Usage:\tquery <ql> <query>\tRun a query using the given QL.\n' +
	'\tload <path>\t\tLoad the given API Docs JS/JSON file.\n' +
	'\thelp\t\t\tDisplay this help.';

console.log(USAGE);

var c = repl.start({
	eval: function (cmd, context, filename, callback) {
		var match, ql;

		cmd = cmd.replace(/\((.*)\n\)/, '$1');

		if (cmd.length === 0) {
			return callback('Enter a command');
		}

		if (cmd === 'help') {
			return callback(USAGE);
		}

		match = cmd.match(/^load (.+)$/);

		if (match !== null) {

			return spy.load(path.resolve(match[1]), function (err, dcs) {

				if (err) {
					return callback(err);
				}

				docs = dcs;

				return callback(util.format('Loaded: %s', match[1]));
			});
		}

		match = cmd.match(/^query ([a-z]+)(?: (.+))?$/);

		if (match === null) {
			return callback('Unknown command');
		}

		ql = match[2] ? match[1] : undefined;
		cmd = match[2] ? match[2] : match[1];

		if (!docs) {
			return callback(util.format('Load a source first.'));
		}

		docs.query(cmd, {
			ql: ql
		}, function (err, res) {

			if (err) {
				callback('\n' + err.toString() + '\n');
			} else {
				callback(null, res);
			}

			// console.log('cb', [err, res]);

		});
	}
});

c.usage = USAGE;
