var path = require('path'),
	fs = require('fs'),
	pkginfo = require('pkginfo')(module),
	async = require('async'),
	jf = require('jsonfile'),
	_ = require('lodash');

exports.load = load;
exports.query = query;

function load(source, callback) {

	var docs = {
		source: {},
		release: function () {
			this.source = {};
		},
		query: function (query, opts, callback) {

			if (arguments.length === 2) {
				callback = opts;
				opts = {};
			}

			var self = this;

			// smart default for QL
			if (!opts.ql) {

				if (query.charAt(0) === '.') {
					opts.ql = 'jspath';
				} else if (query.charAt(0) === '/') {
					opts.ql = 'path';
				} else {
					opts.ql = 'jspath';
				}
			}

			normalizeQL(opts.ql, function (err, ql) {

				if (err) {
					return callback(err);
				}

				ql.query(self.source, query, opts, callback);
			});
		}
	};

	if (_.isObject(source) && !_.isArray(source)) {
		docs.source = source;

		return callback(null, docs);
	}

	if (_.isString(source)) {
		var extname = path.extname(source);

		if (extname === '.json') {
			return jf.readFile(source, function (err, source) {

				if (err) {
					return callback(err);
				}

				docs.source = source;

				callback(null, docs);
			});
		}

		try {
			docs.source = require(source);
		} catch (err) {
			return callback(err.message);
		}

		docs.release = function () {
			this.source = {};
			delete require.cache[require.resolve(source)];
		};

		return callback(null, docs);
	}

	callback('Argument must be an object or a path to a JS or JSON file.');
}

function query(source, query, opts, callback) {

	if (arguments.length === 3) {
		callback = opts;
		opts = {};
	}

	load(source, function (err, docs) {

		if (err) {
			return callback(err);
		}

		docs.query(query, opts, function (err, res) {

			callback(err, res);

			docs.release();
		});

	});
}

function normalizeQL(ql, callback) {

	if (_.isObject(ql) && _.isFunction(ql.query)) {
		return callback(null, ql);
	}

	if (typeof ql !== 'string' || ql === '') {
		return callback('QL must be an object with query-method, the path to a QL module or the name of a built-in QL.');
	}

	fs.exists(ql, function (exists) {

		if (!exists) {
			ql = path.join(__dirname, 'ql', ql + '.js');
		}

		try {
			ql = require(ql);
		} catch (err) {
			return callback(err.message);
		}

		return callback(null, ql);
	});
}
